@@ -1,12 +1,22 @@
 # -*- coding: utf-8 -*-
+#
+# IMPORTANT:
+# Please see https://github.com/py-moneyed/py-moneyed/issues/22#issuecomment-447059971 before making changes here.
 
 from __future__ import unicode_literals
-from decimal import Decimal, ROUND_HALF_EVEN
+
+import warnings
+from decimal import ROUND_HALF_EVEN, Decimal
+
 import moneyed
 
 DEFAULT = "DEFAULT"
 
 
+warnings.warn('This module and all its contents is deprecated in favour of new moneyed.l10n.format_money.',
+              DeprecationWarning)
+
+
 class CurrencyFormatter(object):
 
     sign_definitions = {}
@@ -205,9 +215,8 @@ _format("nn_NO", group_size=3, group_separator=" ", decimal_point=",",
 # foreign or local currency signs for one reason or another differ
 # from the norm.
 
-# There may be errors here, they have been entered manually. Please
-# fork and fix if you find errors.
-# Code lives here (2011-05-08): https://github.com/limist/py-moneyed
+# READ ME FIRST:
+#  https://github.com/py-moneyed/py-moneyed/issues/22#issuecomment-447059971
 
 _sign(DEFAULT, moneyed.AED, prefix='د.إ')
 _sign(DEFAULT, moneyed.AFN, suffix='؋')
