@@ -7,12 +7,17 @@ from copy import deepcopy
 from decimal import Decimal
 
 import pytest  # Works with less code, more consistency than unittest.
+from babel.core import get_global
 
 from moneyed.classes import (CURRENCIES, DEFAULT_CURRENCY, PYTHON2, USD,
                              Currency, Money, MoneyComparisonError,
-                             force_decimal, get_currency)
+                             force_decimal, get_currencies_of_country,
+                             get_currency, list_all_currencies)
 from moneyed.localization import format_money
 
+if not PYTHON2:
+    unicode = str  # Avoid UndefinedName flake8 errors in Python3
+
 
 class CustomDecimal(Decimal):
     """Test class to ensure Decimal.__str__ is not
@@ -29,30 +34,27 @@ class TestCurrency:
         self.default_curr = CURRENCIES[self.default_curr_code]
 
     def test_init(self):
-        usd_countries = CURRENCIES['USD'].countries
         US_dollars = Currency(
             code='USD',
             numeric='840',
-            name='US Dollar',
-            countries=['AMERICAN SAMOA',
-                       'BRITISH INDIAN OCEAN TERRITORY',
-                       'ECUADOR',
-                       'GUAM',
-                       'MARSHALL ISLANDS',
-                       'MICRONESIA',
-                       'NORTHERN MARIANA ISLANDS',
-                       'PALAU',
-                       'PUERTO RICO',
-                       'TIMOR-LESTE',
-                       'TURKS AND CAICOS ISLANDS',
-                       'UNITED STATES',
-                       'UNITED STATES MINOR OUTLYING ISLANDS',
-                       'VIRGIN ISLANDS (BRITISH)',
-                       'VIRGIN ISLANDS (U.S.)'])
+            sub_unit=100,
+            name='United States Dollar',  # NB deliberately not official name
+            countries=['UNITED STATES'])
         assert US_dollars.code == 'USD'
-        assert US_dollars.countries == usd_countries
-        assert US_dollars.name == 'US Dollar'
+        assert US_dollars.countries == ['UNITED STATES']
+        assert US_dollars.name == 'United States Dollar'
         assert US_dollars.numeric == '840'
+        assert US_dollars.sub_unit == 100
+
+    def test_name(self):
+        assert USD.name == "US Dollar"
+
+    def test_countries(self):
+        assert CURRENCIES['AED'].countries == ['UNITED ARAB EMIRATES']
+
+    def test_get_name(self):
+        assert USD.get_name('es') == 'dólar estadounidense'
+        assert USD.get_name('en_GB', count=10) == 'US dollars'
 
     def test_repr(self):
         assert str(self.default_curr) == self.default_curr_code
@@ -75,6 +77,12 @@ class TestCurrency:
         assert get_currency(iso='840') == USD
         assert get_currency(iso=840) == USD
 
+    def test_get_currencies_of_country(self):
+        assert get_currencies_of_country("IN")[0] == Currency('INR')
+        assert get_currencies_of_country("iN")[0] == Currency('INR')
+        assert get_currencies_of_country("BT") == [Currency('BTN'), Currency('INR')]
+        assert get_currencies_of_country("XX") == []
+
 
 class TestMoney:
 
@@ -107,20 +115,27 @@ class TestMoney:
         assert one_million_dollars.amount == self.one_million_decimal
 
     def test_repr(self):
-        assert repr(self.one_million_bucks) == '<Money: 1000000 USD>'
-        assert repr(Money(Decimal('2.000'), 'PLN')) == '<Money: 2.000 PLN>'
+        assert repr(self.one_million_bucks) == "Money('1000000', 'USD')"
+        assert repr(Money(Decimal('2.000'), 'PLN')) == "Money('2.000', 'PLN')"
         m_1 = Money(Decimal('2.00'), 'PLN')
         m_2 = Money(Decimal('2.01'), 'PLN')
         assert repr(m_1) != repr(m_2)
 
+    def test_eval_from_repr(self):
+        m = Money('1000', 'USD')
+        assert m == eval(repr(m))
+
     def test_str(self):
-        one_million_pln = Money('1000000', 'PLN')
         if PYTHON2:
-            assert str(one_million_pln) == 'PLN1,000,000.00'
-            assert str(self.one_million_bucks) == 'USD1,000,000.00'
+            # Bytestring uses a simpler fallback to avoid unicode chars
+            assert str(self.one_million_bucks) == 'USD1,000,000'
+
+        # Conversion to text use default locale, so results vary
+        # depending on system setup. Just assert that we don't crash.
+        if PYTHON2:
+            assert isinstance(unicode(self.one_million_bucks), unicode)
         else:
-            assert str(one_million_pln) == '1,000,000.00 zł'
-            assert str(self.one_million_bucks) == 'US$1,000,000.00'
+            assert isinstance(str(self.one_million_bucks), str)
 
     def test_hash(self):
         assert self.one_million_bucks in set([self.one_million_bucks])
@@ -172,6 +187,11 @@ class TestMoney:
         with pytest.raises(TypeError):
             Money(1000) - 123
 
+    def test_rsub_non_money(self):
+        assert 0 - Money(1, currency=self.USD) == Money(-1, currency=self.USD)
+        with pytest.raises(TypeError):
+            assert 1 - Money(3, currency=self.USD) == Money(-2, currency=self.USD)
+
     def test_mul(self):
         x = Money(amount=111.33, currency=self.USD)
         assert 3 * x == Money(333.99, currency=self.USD)
@@ -209,6 +229,12 @@ class TestMoney:
         y = 2
         assert x / y == Money(amount=25, currency=self.USD)
 
+    def test_rdiv_by_non_Money(self):
+        x = 2
+        y = Money(amount=50, currency=self.USD)
+        with pytest.raises(TypeError):
+            assert x / y == Money(amount=25, currency=self.USD)
+
     def test_div_float_warning(self):
         # This should be changed to TypeError exception after deprecation period is over.
         with warnings.catch_warnings(record=True) as warning_list:
@@ -311,6 +337,10 @@ class TestMoney:
             x = Money(amount='3.5', currency=self.USD)
             assert x.round(0) == Money(amount=4, currency=self.USD)
 
+    def test_get_sub_unit(self):
+        m = Money(amount=123, currency=self.USD)
+        assert m.get_amount_in_sub_unit() == 12300
+
     def test_arithmetic_operations_return_real_subclass_instance(self):
         """
         Arithmetic operations on a subclass instance should return instances in the same subclass
@@ -375,3 +405,16 @@ class ExtendedMoney(Money):
 
     def do_my_behaviour(self):
         pass
+
+
+def test_all_babel_currencies():
+    missing = sorted(list(set(get_global('all_currencies').keys()) - set(CURRENCIES.keys())))
+    assert missing == [], \
+        'The following currencies defined in Babel are missing: ' + ', '.join(missing)
+
+
+def test_list_all_currencies():
+    all_currencies = list_all_currencies()
+    assert len(all_currencies) > 100
+    assert [c.code for c in all_currencies[0:3]] == ['ADP', 'AED', 'AFA']
+    assert all(isinstance(c, Currency) for c in all_currencies)
